<?php
/**
 * Admin Settings Pages Helper.
 *
 * @package    BuddyCircles
 * @subpackage Admin
 * @copyright  Copyright (c) 2018, BuddyDev.Com
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Ravi Sharma, Brajesh Singh
 * @since      1.0.0
 */

namespace PressThemes\BuddyCircles\Admin;

use \Press_Themes\PT_Settings\Page;

// Exit if file accessed directly over web.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class Admin_Settings
 */
class Admin_Settings {

	/**
	 * Admin Menu slug
	 *
	 * @var string
	 */
	private $menu_slug;

	/**
	 * Used to keep a reference of the Page, It will be used in rendering the view.
	 *
	 * @var \Press_Themes\PT_Settings\Page
	 */
	private $page;

	/**
	 * Boot settings
	 */
	public static function boot() {
		$self = new self();
		$self->setup();
	}

	/**
	 * Setup settings
	 */
	public function setup() {

		$this->menu_slug = 'buddycircles';

		add_action( 'admin_init', array( $this, 'init' ) );
		add_action( 'admin_menu', array( $this, 'add_menu' ) );
	}

	/**
	 * Show/render the setting page
	 */
	public function render() {
		$this->page->render();
	}

	/**
	 * Is it the setting page?
	 *
	 * @return bool
	 */
	private function needs_loading() {

		global $pagenow;

		// We need to load on options.php otherwise settings won't be reistered.
		if ( 'options.php' === $pagenow ) {
			return true;
		}

		if ( isset( $_GET['page'] ) && $_GET['page'] === $this->menu_slug ) {
			return true;
		}

		return false;
	}

	/**
	 * Initialize the admin settings panel and fields
	 */
	public function init() {

		if ( ! $this->needs_loading() ) {
			return;
		}

		$page = new Page( 'bcircles_settings', __( 'BuddyCircles', 'buddycircles' ) );

		// General settings tab.
		$general = $page->add_panel( 'general', _x( 'General', 'Admin settings panel title', 'buddycircles' ) );

		$section_general = $general->add_section( 'settings', _x( 'General Settings', 'Admin settings section title', 'buddycircles' ) );

		$parent_options = array(
			''        => __( 'Top Level Component', 'buddycircles' ),
			'friends' => __( 'Friends Tab', 'buddycircles' ),
		);

		$defaults = bcircles_get_default_options();

		// add the follow option in future.
		$fields = array(
			array(
				'name'    => 'slug',
				'label'   => _x( 'Slug', 'Admin settings', 'buddycircles' ),
				'type'    => 'text',
				'default' => $defaults['slug'],
			),
			array(
				'name'    => 'who_can_create',
				'label'   => _x( 'Who can create', 'Admin settings', 'buddycircles' ),
				'type'    => 'multicheck',
				'options' => $this->get_roles(),
				'default' => $defaults['who_can_create'],
			),
			array(
				'name'    => 'parent_component',
				'label'   => _x( 'Keep Circle Under', 'Admin settings', 'buddycircles' ),
				'type'    => 'select',
				'options' => $parent_options,
				'default' => $defaults['parent_component'],
			),
			array(
				'name'    => 'circle_member_type',
				'label'   => _x( 'Who Can be added to the circle?', 'Admin settings', 'buddycircles' ),
				'type'    => 'select',
				'options' => array(
					'all'          => __( 'All members', 'buddycircles' ),
					'friends_only' => __( 'Friends only', 'buddycircles' ),
				),
				'default' => $defaults['circle_member_type'],
			),
			array(
				'name'    => 'privacy',
				'label'   => _x( 'Circle privacy', 'Admin settings', 'buddycircles' ),
				'type'    => 'multicheck',
				'options' => bcircles_get_privacies(),
				'default' => $defaults['privacy'],
			),
			array(
				'name'    => 'create_default',
				'label'   => _x( 'Create default circles for new users', 'Admin settings', 'buddycircles' ),
				'type'    => 'checkbox',
				'default' => $defaults['create_default'],
			),
			array(
				'name'    => 'default_circles',
				'label'   => _x( 'Default circles', 'Admin settings', 'buddycircles' ),
				'type'    => 'textarea',
				'default' => $defaults['default_circles'],
				'desc'    => __( 'Please enter names separated by "|". These circles will be created on account activation', 'buddycircles' ),
			),
			array(
				'name'    => 'default_circles_privacy',
				'label'   => _x( 'Privacy for default circles', 'Admin settings', 'buddycircles' ),
				'type'    => 'radio',
				'options' => bcircles_get_privacies(),
				'default' => $defaults['default_circles_privacy'],
				'desc'    => __( 'Selected privacy will be attached with the default circles at the time of creation.Please make sure default privacy is enabled under circle privacy options.', 'buddycircles' ),
			),
		);

		$section_general->add_fields( $fields );

		$notification_panel = $page->add_panel( 'notifications', _x( 'Notifications', 'Admin settings panel title', 'buddycircles' ) );

		$section_notifications = $notification_panel->add_section( 'notification', _x( 'Notifications Settings', 'Admin settings section title', 'buddycircles' ) );

		$section_notifications->add_fields(
			array(
				array(
					'name'    => 'enabled_notification_types',
					'label'   => _x( 'Enabled Notification Types:', 'Admin settings', 'buddycircles' ),
					'type'    => 'multicheck',
					'options' => array(
						'local' => __( 'Local notification', 'buddycircles' ),
						'email' => __( 'Email notification', 'buddycircles' ),
					),
					'default' => $defaults['enabled_notification_types'],
				),

				array(
					'name'    => 'notify_user_after_adding',
					'label'   => _x( 'Notify user when they get added to a circle?', 'Admin settings', 'buddycircles' ),
					'type'    => 'radio',
					'options' => array(
						1 => __( 'Yes', 'buddycircles' ),
						0 => __( 'No', 'buddycircles' ),
					),
					'default' => $defaults['notify_user_after_adding'],
				),


				array(
					'name'    => 'user_added_to_circle_email_subject',
					'label'   => _x( 'Email subject', 'Admin settings', 'buddycircles' ),
					'type'    => 'text',
					'desc'    => __( 'Notification message subject for user being added to a circle.', 'buddycircles' ),
					'default' => $defaults['user_added_to_circle_email_subject'],
				),
				array(
					'name'    => 'user_added_to_circle_email_message',
					'label'   => _x( 'Message', 'Admin settings', 'buddycircles' ),
					'type'    => 'rawtext',
					'desc'    => __( 'Message on user addition to new circle.', 'buddycircles' ),
					'default' => $defaults['user_added_to_circle_email_message'],
				),
				array(
					'name'    => 'notify_user_after_removal',
					'label'   => _x( 'Notify user when they get removed from a circle?', 'Admin settings', 'buddycircles' ),
					'type'    => 'radio',
					'options' => array(
						1 => __( 'Yes', 'buddycircles' ),
						0 => __( 'No', 'buddycircles' ),
					),
					'default' => $defaults['notify_user_after_removal'],
				),
				array(
					'name'    => 'user_removed_from_circle_email_subject',
					'label'   => _x( 'Email subject', 'Admin settings', 'buddycircles' ),
					'type'    => 'text',
					'desc'    => __( 'Email notification subject for user removal from circle', 'buddycircles' ),
					'default' => $defaults['user_removed_from_circle_email_subject'],
				),
				array(
					'name'    => 'user_removed_from_circle_email_message',
					'label'   => _x( 'Message', 'Admin settings', 'buddycircles' ),
					'type'    => 'rawtext',
					'desc'    => __( 'Email notification subject for user removal from circle', 'buddycircles' ),
					'default' => $defaults['user_removed_from_circle_email_message'],
				),
			)
		);

		$misc_panel = $page->add_panel( 'misc', _x( 'Miscellaneous', 'Admin settings panel title', 'buddycircles' ) );

		$section_misc = $misc_panel->add_section( 'misc', _x( 'Miscellaneous', 'Admin settings section title', 'buddycircles' ) );
		$section_misc->add_fields(
			array(
				array(
					'name'    => 'enable_directory_tabs',
					'label'   => _x( 'Eanble circles as tab on members directory', 'buddycircles' ),
					'type'    => 'radio',
					'options' => array(
						1 => __( 'Yes', 'buddycircles' ),
						0 => __( 'No', 'buddycircles' ),
					),
					'default' => $defaults['enable_directory_tabs'],
					'desc'    => __( 'Note: Currently only works with BuddyPress legacy template pack.', 'buddycircles' ),
				),
			)
		);

		do_action( 'bcircles_admin_settings_page', $page );

		$this->page = $page;

		// allow enabling options.
		$page->init();
	}

	/**
	 * Add Menu
	 */
	public function add_menu() {

		add_options_page(
			_x( 'BuddyCircles', 'Admin settings page title', 'buddycircles' ),
			_x( 'BuddyCircles', 'Admin settings menu label', 'buddycircles' ),
			'manage_options',
			$this->menu_slug,
			array( $this, 'render' )
		);
	}

	/**
	 * Get roles
	 *
	 * @return array
	 */
	private function get_roles() {
		$roles = get_editable_roles();

		$user_roles = array(
			'all' => __( 'All Members', 'buddycircles' ),
		);

		foreach ( $roles as $role => $detail ) {
			$user_roles[ $role ] = $detail['name'];
		}

		return $user_roles;
	}
}
