<?php
/**
 * Circle component
 *
 * @package    BuddyCircles
 * @subpackage BuddyPress
 */

namespace PressThemes\BuddyCircles\BuddyPress;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class Circle_Component
 *
 * Used when circle is implemented as top level tab.
 */
class Circle_Component extends \BP_Component {

	/**
	 * The constructor.
	 */
	public function __construct() {

		parent::start(
			'circle',
			__( 'Circle', 'buddycircles' ),
			untrailingslashit( plugin_dir_path( __FILE__ ) )
		);

		buddypress()->active_components[ $this->id ] = 1;
	}

	/**
	 * Setup globals
	 *
	 * @param array $args Globals array.
	 */
	public function setup_globals( $args = array() ) {
		$slug = bcircles_get_circle_slug();

		if ( ! defined( 'BUDDYCIRCLES_SLUG' ) ) {
			define( 'BUDDYCIRCLES_SLUG', $slug );
		}

		$args = array(
			'slug'                  => BUDDYCIRCLES_SLUG,
			'root_slug'             => BUDDYCIRCLES_SLUG,
			'has_directory'         => false,
			'notification_callback' => 'bcircles_format_notifications',
			'search_string'         => __( 'Search Circles...', 'buddycircles' ),
			'global_tables'         => array(),
		);

		parent::setup_globals( $args );
	}

	/**
	 * Registers navigation
	 *
	 * @param array $main_nav Main nav array.
	 * @param array $sub_nav  Sub nav array.
	 *
	 * @return array|void
	 */
	public function register_nav( $main_nav = array(), $sub_nav = array() ) {

		if ( ! $this->is_enabled_top_nav() ) {
			parent::register_nav( $main_nav, $sub_nav );

			return;
		}

		$screen = new Circle_Screens();

		// Note: Allow positioning form the backend.
		$main_nav = array(
			'name'                => __( 'Circle', 'buddycircle' ),
			'slug'                => $this->slug,
			'position'            => 74,
			'screen_function'     => array( $screen, 'circles' ),
			'default_subnav_slug' => 'all-circles',
			'item_css_id'         => $this->id,
		);

		$sub_nav[] = array(
			'name'            => __( 'Circles', 'buddycircles' ),
			'slug'            => 'all-circles',
			'parent_slug'     => $this->get_parent_slug(),
			'screen_function' => array( $screen, 'circles' ),
			'position'        => 10,
		);

		if ( bcircles_is_bp_rewrite_enabled() ) {
			parent::register_nav( $main_nav, $sub_nav );
		} else {
			return array(
				'main_nav' => $main_nav,
				'sub_nav'  => $sub_nav,
			);
		}
	}

	/**
	 * Setup navigation
	 *
	 * @param array $main_nav Main nav array.
	 * @param array $sub_nav  Sub nav array.
	 */
	public function setup_nav( $main_nav = array(), $sub_nav = array() ) {

		if ( ! $this->is_enabled_top_nav() ) {
			parent::setup_nav( $main_nav, $sub_nav );

			return;
		}

		$privacy = is_super_admin() ? '' : bcircles_get_option( 'privacy' );

		$total_circles = bcircles_get_total_circle_count(
			array(
				'user_id' => bp_displayed_user_id(),
				'privacy' => $privacy,
			)
		);

		$total_circles = empty( $total_circles ) ? '' : $total_circles;
		$class         = ( 0 === $total_circles ) ? 'no-count' : 'count';

		$main_nav_name = sprintf(
			_x( 'Circle %s', 'Profile screen nav', 'buddycircles' ),
			sprintf(
				'<span class="%s">%s</span>',
				esc_attr( $class ),
				esc_html( $total_circles )
			)
		);

		if ( bcircles_is_bp_rewrite_enabled() ) {
			$this->main_nav['name'] = $main_nav_name;

			parent::setup_nav( $main_nav, $sub_nav );

			return;
		}

		$nav_items = $this->register_nav( $main_nav, $sub_nav );

		$nav_items['main_nav']['name'] = $main_nav_name;

		foreach ( $nav_items['sub_nav'] as $index => $sub_nav ) {
			$parent_url = bcircles_get_displayed_user_url( array( $sub_nav['parent_slug'] ) );

			$nav_items['sub_nav'][ $index ]['parent_url'] = $parent_url;
		}

		parent::setup_nav( $nav_items['main_nav'], $nav_items['sub_nav'] );
	}

	/**
	 * Set up the Toolbar
	 *
	 * Admin bar
	 *
	 * @param array $nav Admin nav.
	 */
	public function setup_admin_bar( $wp_admin_nav = array() ) {

		if ( ! $this->is_enabled_top_nav() || ! is_user_logged_in() ) {
			return;
		}

		$wp_admin_nav[] = array(
			'parent'   => buddypress()->my_account_menu_id,
			'id'       => 'my-account-' . $this->id . '-' . $this->slug,
			'title'    => __( 'Circles', 'buddycircles' ),
			'href'     => bcircles_get_loggedin_user_url( array( $this->get_parent_slug() ) ),
			'position' => 5,
		);

		parent::setup_admin_bar( $wp_admin_nav );
	}

	/**
	 * Sets up the title for pages and <title>
	 */
	public function setup_title() {

		$bp = buddypress();
		// Note: May not be needed and needs a relook.
		if ( bp_is_friends_component() && bp_is_current_action( bcircles_get_circle_slug() ) ) {

			if ( bp_is_my_profile() && ! bp_is_single_item() ) {
				$bp->bp_options_title = __( 'Circles', 'buddycircles' );
			} elseif ( ! bp_is_my_profile() && ! bp_is_single_item() ) {

				$bp->bp_options_avatar = bp_core_fetch_avatar(
					array(
						'item_id' => bp_displayed_user_id(),
						'type'    => 'thumb',
						'alt'     => sprintf( __( 'Profile picture of %s', 'buddycircles' ), bp_get_displayed_user_fullname() ),
					)
				);

				$bp->bp_options_title = bp_get_displayed_user_fullname();
			}
		}

		parent::setup_title();
	}

	/**
	 * Checks if top navigation enabled or not.
	 *
	 * @return bool
	 */
	public function is_enabled_top_nav() {
		return bcircles_get_parent_component_slug() ? false : true;
	}

	/**
	 * Retrieves parent slug
	 *
	 * @return string
	 */
	private function get_parent_slug() {
		$slug = bcircles_get_parent_component_slug();
		$slug = $slug ? $slug : $this->slug;

		return $slug;
	}
}
