<?php
/**
 * Core functions file
 *
 * @package BuddyCircles
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use PressThemes\BuddyCircles\Models\Circle;
use PressThemes\BuddyCircles\Models\Circle_Member;
use PressThemes\BuddyCircles\Schema\Schema;

/**
 * Get circle information
 *
 * @param int $circle_id Circle or circle id.
 *
 * @return Circle|false
 */
function bcircles_get_circle( $circle_id ) {

	if ( $circle_id instanceof Circle ) {
		return $circle_id;
	}

	if ( is_object( $circle_id ) ) {
		$circle_id = isset( $circle_id->id ) ? $circle_id->id : 0;
	}

	// check from cache
	$circle = wp_cache_get( "circle_{$circle_id}", 'bcircles' );

	if ( false === $circle ) {
		global $wpdb;

		$table  = Schema::table( 'circles' );
		$circle = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$table} WHERE id=%d", $circle_id ) );

		if ( $circle ) {
			wp_cache_set( "circle_{$circle_id}", $circle, 'bcircles' );
		}
	}

	if ( $circle ) {
		return new Circle( get_object_vars( $circle ) );
    }

	return false;
}

/**
 * Create new circle
 *
 * @param array $args Args.
 *  @type int $id
 *  @type int $user_id
 *  @type string $name
 *  @type string $privacy
 *
 * @return bool|Circle
 */
function bcircles_create_circle( $args ) {

	$circle = new Circle( $args );

	if ( ! $circle->save() ) {
		return false;
	}

	wp_cache_delete( "user_circles_{$circle->user_id}", 'bcircles' );
	wp_cache_delete( "user_{$circle->user_id}_all_circles_users", 'bcircles' );

	return $circle;
}

/**
 * Delete the circle.
 *
 * @param int|Circle $circle circle id or object.
 *
 * @return bool
 */
function bcircles_delete_circle( $circle ) {

	$circle = bcircles_get_circle( $circle );

	if ( ! $circle ) {
		return false;
	}

	$circle->delete();

	bcircles_circle_cache_prune( $circle );

	return true;
}

/**
 * Update new circle
 *
 * @param array $args
 *  @type int $id
 *  @type int $user_id
 *  @type string $name
 *  @type string $privacy
 *
 * @return bool
 */
function bcircles_update_circle( $args ) {

	if ( ! isset( $args['id'] ) || ! is_numeric( $args['id'] ) ) {
		return false;
	}

	$circle = bcircles_get_circle( $args['id'] );

	if ( empty( $circle ) ) {
		return false;
	}

	if ( isset( $args['label'] ) ) {
		$circle->label = $args['label'];
	}

	if ( isset( $args['user_id'] ) ) {
		$circle->user_id = $args['user_id'];
	}

	if ( isset( $args['privacy'] ) ) {
		$circle->privacy = $args['privacy'];
	}

	bcircles_circle_cache_prune( $circle );

	return $circle->save();
}

/**
 * Check existence
 *
 * @param array $args Args.
 *
 * @return bool
 */
function bcircles_circle_exists( $args ) {
	return Circle::exists( $args );
}

/**
 * Check if label is duplicate
 *
 * @param array $args Args.
 *
 * @return bool
 */
function bcircles_is_label_duplicate( $args ) {
	return Circle::is_label_duplicate( $args );
}

/**
 * Get total circle
 *
 * @param array $args Args.
 *
 * @return string|null
 */
function bcircles_get_total_circle_count( $args ) {
	return Circle::get_total( $args );
}

/**
 * Get circles user ids
 *
 * @param array $circle_ids Circles user id.
 *
 * @return array
 */
function bcircles_get_users_for_all_circles( $circle_ids = array() ) {

	if ( empty( $circle_ids ) ) {
		return array();
	}

	$users_ids = array();
	foreach ( $circle_ids as $circle_id ) {
		$users_ids = array_merge( $users_ids, bcircles_get_circle_users( $circle_id ) );
	}

	return  array_unique( $users_ids );
}

/**
 * Get all users in the given circle
 *
 * @param int $circle_id Circle id.
 *
 * @return array
 */
function bcircles_get_circle_users( $circle_id ) {
	if ( is_object( $circle_id ) && isset( $circle_id->id ) ) {
		$circle_id = $circle_id->id;
	}

	$circle_users = wp_cache_get( "circle_{$circle_id}_users", 'bcircles' );

	if ( false !== $circle_users ) {
		return $circle_users;
	}

	$circle_users = Circle_Member::get_by( array(
		'circle_id' => $circle_id,
	) );

	if ( empty( $circle_users ) ) {
		$user_ids = array();
	} else {
		$user_ids = wp_list_pluck( $circle_users, 'user_id' );
	}

	wp_cache_set( "circle_{$circle_id}_users", $user_ids, 'bcircles' );

	return $user_ids;
}

/**
 * Add user to circle
 *
 * @param int $user_id User id.
 * @param int $circle_id Circle id.
 *
 * @return bool|Circle_Member
 */
function bcircles_add_circle_user( $user_id, $circle_id ) {

	$user_circle = new Circle_Member( array(
		'user_id'   => $user_id,
		'circle_id' => $circle_id,
	) );

	if ( ! $user_circle->save() ) {
		return false;
	}

	wp_cache_delete( "circle_{$circle_id}_users", 'bcircles' );
	wp_cache_delete( "user_{$user_id}_all_circles_users", 'bcircles' );

	do_action( 'bcircles_circle_user_added', $user_id, $circle_id );

	return $user_circle;
}

/**
 * Remove user from circle
 *
 * @param int $user_id User id.
 * @param int $circle_id Circle id.
 *
 * @return bool
 */
function bcircles_remove_circle_user( $user_id, $circle_id ) {

	$user_deleted = Circle_Member::delete_by( array(
		'user_id'   => $user_id,
		'circle_id' => $circle_id,
	) );

	if ( ! $user_deleted ) {
		return false;
	}

	wp_cache_delete( "circle_{$circle_id}_users", 'bcircles' );
	wp_cache_delete( "user_circles_{$user_id}", 'bcircles' );
	wp_cache_delete( "user_{$user_id}_all_circles_users", 'bcircles' );

	do_action( 'bcircles_circle_user_removed', $user_id, $circle_id );

	return $user_deleted;
}

/**
 * Remove user from all circle
 *
 * @param int $user_id User id.
 *
 * @return bool
 */
function bcircles_remove_user( $user_id ) {

	// Remove user caches
	// we also need to clear cache for all circles where this user was a members.
	$circles = bcircles_get_user_circles( $user_id, false, false);

	if ( $circles ) {
		foreach ( $circles as $circle ) {
			$circle = bcircles_get_circle( $circle );

			if ( empty( $circle ) ) {
				continue;
			}

			bcircles_circle_cache_prune( $circle );
		}
	}

	wp_cache_delete( "user_circles_{$user_id}", 'bcircles' );
	wp_cache_delete( "user_{$user_id}_all_circles_users", 'bcircles' );


	return Circle_Member::delete_by( array(
		'user_id' => $user_id,
	) );
}

/**
 * Get user circles
 *
 * @param int $user_id User id.
 *
 * @return bool|mixed
 */
function bcircles_get_user_circles( $user_id , $cache_count = true, $cache_members = false ) {

	$circles = wp_cache_get( "user_circles_{$user_id}", 'bcircles' );

	if ( false === $circles ) {
		$circles = Circle::get_by( array(
			'user_id' => $user_id,
			'privacy' => bcircles_get_option( 'privacy' ),
			'fields' => 'id',
		) );

		if ( empty( $circles ) ) {
			return array();
		}

		$circles = wp_list_pluck( $circles, 'id' );
		bcircles_cache_circles( $circles, $cache_count , $cache_members );
		wp_cache_set( "user_circles_{$user_id}", $circles, 'bcircles' );
	}

	return $circles;
}

function bcircles_get_circle_objects( $circle_ids ) {
	if( empty( $circle_ids ) ) {
		return array();
	}

	$circles = array();

	foreach( $circle_ids as $circle_id ) {
		$circles[$circle_id] = bcircles_get_circle( $circle_id );
	}

	return $circles;
}

/**
 * Is the other user in the $user_id's circle?
 *
 *  Circles count between two user
 *
 * @param int $user_id User id.
 * @param int $other_user_id Other user id.
 *
 * @return int
 */
function bcircles_has_user_in_circles( $user_id, $other_user_id ) {
	return bcircles_get_circle_count_between_users( $user_id, $other_user_id );
}

/**
 * Get users from all circles of the given user.
 *
 * @param int $user_id User id.
 *
 * @return array|bool|mixed
 */
function bcircles_get_all_circle_users( $user_id ) {

	$users = wp_cache_get( "user_{$user_id}_all_circles_users", 'bcircles' );

	if ( false === $users ) {
		$users = Circle_Member::get_user_circles_users( $user_id );
		wp_cache_set( "user_{$user_id}_all_circles_users", $users, 'bcircles' );
	}

	return $users;
}

/**
 * User circles count
 *
 * @param int $user_id User id.
 * @param int $other_user_id Other user id.
 *
 * @return int
 */
function bcircles_get_circle_count_between_users( $user_id, $other_user_id ) {
	return count( bcircles_get_circles_between_users( $user_id, $other_user_id ) );
}

/**
 * Circle users in user circles
 *
 * @param int $user_id User id.
 * @param int $other_user_id Other user id.
 *
 * @return array
 */
function bcircles_get_circles_between_users( $user_id, $other_user_id ) {

	$user_in_user_circles = bcircles_get_all_circle_users( $user_id );

	if ( isset( $user_in_user_circles[ $other_user_id ] ) ) {
		return $user_in_user_circles[ $other_user_id ];
	}

	return array();
}

/**
 * Get member circles
 *
 * @param int $user_id User id.
 * @param int $member_id Member id.
 *
 * @return array|null|object
 */
function bcircles_get_member_circles( $user_id, $member_id ) {
	return Circle_Member::get_member_circles( $user_id, $member_id );
}

/**
 * Get default circles to create.
 *
 * @return array
 */
function bcircles_get_default_circles() {

    $circles = (string) bcircles_get_option( 'default_circles' );

    $circles = explode( '|', $circles );

    if( empty( $circles ) ) {
        return $circles;
    }

    $circles = array_map( 'trim', $circles );

    return $circles;
}

/**
 * Circle privacies
 *
 * @return array
 */
function bcircles_get_privacies() {
	return array(
		'public'       => __( 'Public', 'buddycircles' ),
		'private'      => __( 'Private', 'buddycircles' ),
		'friends_only' => __( 'Friends only', 'buddycircles' ),
	);
}

/**
 * Get setting
 *
 * @param string $setting Setting name.
 * @param mixed  $default default value.
 *
 * @return mixed
 */
function bcircles_get_option( $setting = '', $default = null ) {

	$settings = get_option( 'bcircles_settings', bcircles_get_default_options() );

	if ( isset( $settings[ $setting ] ) ) {
		return $settings[ $setting ];
	}

	return $default;
}

/**
 * Get default options.
 *
 * @return array
 */
function bcircles_get_default_options() {

	$default_circles = join( '|', array(
		__( 'Close Buddies', 'buddycircles' ),
		__( 'School friends', 'buddycircles' ),
		__( 'College friends', 'buddycircles' ),
		__( 'Family', 'buddycircles' ),
	) );

	$added_message = <<<EOL
Hello {receiver_name}

{user_name} has added you in their circle {circle_name}.	
EOL;

	$deleted_message = <<<EOL
Hello {receiver_name}

{user_name} has removed you from their circle {circle_name}.
EOL;

	$defaults = array(
		'slug'                                   => 'circle',
		'who_can_create'                         => array(
			'all' => 'all',
		),
		'parent_component'                       => '',
		'circle_member_type'                     => 'all',
		'privacy'                                => array(
			'public' => 'public',
		),
		'create_default'                         => '',
		'default_circles'                        => $default_circles,
		'default_circles_privacy'                => 'public',
		'notify_user_after_adding'               => 0,
		'notify_user_after_removal'              => 0,
		'enabled_notification_types'             => array(),
		'user_added_to_circle_email_subject'     => __( '{user_name} has added you in their circle', 'buddycircles' ),
		'user_added_to_circle_email_message'     => $added_message,
		'user_removed_from_circle_email_subject' => __( '{user_name} has removed you from their circle', 'buddycircles' ),
		'user_removed_from_circle_email_message' => $deleted_message,
		'enable_directory_tabs'                  => 0,
	);

	return $defaults;
}

/**
 * Get parent component specified for the circle.
 *
 * @return string
 */
function bcircles_get_parent_component_slug() {
	return apply_filters( 'bcircles_parent_component_slug', bcircles_get_option( 'parent_component' ) );
}

/**
 * Locate or load a file from theme or fallback to plugin.
 *
 * @param array $templates templates.
 * @param bool  $load load template?
 *
 * @return string|void
 */
function bcircles_locate_template( $templates, $load = false ) {

	$template_base = 'buddycircles/default';

	foreach ( $templates as $key => $template ) {
		$templates[ $key ] = $template_base . '/' . $template; // prefix the base.
	}

	$located = locate_template( $templates, false, false );

	if ( ! $located ) {
		$path = buddycircles()->path . 'templates/';

		foreach ( $templates as $template ) {
			if ( ! is_readable( $path . $template ) ) {
				continue;
			}

			$located = $path . $template;
			break;
		}
	}

	if ( ! $load ) {
		return $located;
	}

	if ( $located ) {
		require $located;
	}
}

/**
 * Checks if BuddyPress supports rewrite.
 *
 * @return bool
 */
function bcircles_is_bp_rewrite_enabled() {

	if ( function_exists( 'bp_core_get_query_parser' ) && 'rewrites' === bp_core_get_query_parser() ) {
		return true;
	}

	return false;
}