<?php
/**
 * Action handler class
 *
 * @package BuddyCircles
 * @subpackage Handlers
 */

namespace PressThemes\BuddyCircles\Handlers;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use PressThemes\BuddyCircles\Models\Circle;
use PressThemes\BuddyCircles\Models\Circle_Member;

/**
 * Class Actions_Handler
 */
class Actions_Handler {

	/**
	 * Class self boot
	 */
	public static function boot() {
		$self = new self();
		$self->setup();
	}

	/**
	 * Setup
	 */
	private function setup() {

		add_action( 'bcircles_circle_user_added', array( $this, 'notify_user_on_add' ), 10, 2 );
		add_action( 'bcircles_circle_user_removed', array( $this, 'notify_user_on_removal' ), 10, 2 );
		// on circle delete.
		add_action( 'bcircles_circle_deleted', array( $this, 'on_circle_delete' ) );

		// On account activation.
		add_action( 'bp_core_activated_user', array( $this, 'add_circles_on_account_activation' ), 0 );
		// On registration.
		add_action( 'user_register', array( $this, 'add_circles_on_registration' ), 0 );

		// on account delete.
		add_action( 'wpmu_delete_user', array( $this, 'on_account_delete' ) );
		add_action( 'delete_user', array( $this, 'on_account_delete' ) );

		add_action( 'bp_template_redirect', array( $this, 'clear_notifications' ) );

		add_action( 'wp_footer', array( $this, 'load_add_user_template' ) );
	}

	/**
	 * On circle delete, delete all users from this circle.
	 *
	 * @param Circle_Member $circle Circle object.
	 */
	public function on_circle_delete( $circle ) {
		Circle_Member::delete_by( array( 'circle_id' => $circle->id ) );
	}

	/**
	 * Notify user when added to circle
	 *
	 * @param int $user_id User id added.
	 * @param int $circle_id Circle id into which added.
	 */
	public function notify_user_on_add( $user_id, $circle_id ) {
		$can_notify = bcircles_get_option( 'notify_user_after_adding' );

		if ( empty( $can_notify ) ) {
			return;
		}

		$enabled_notification_types = (array) bcircles_get_option( 'enabled_notification_types' );

		if ( in_array( 'local', $enabled_notification_types, true ) && bp_is_active( 'notifications' ) ) {
			$this->local_notify( $user_id, $circle_id, 'user_added' );
		}

		if ( in_array( 'email', $enabled_notification_types, true ) ) {
			$this->email_notify( $user_id, $circle_id, 'user_added' );
		}
	}

	/**
	 * Notify user when deleted
	 *
	 * @param int $user_id User id added.
	 * @param int $circle_id Circle id into which added.
	 */
	public function notify_user_on_removal( $user_id, $circle_id ) {
		$can_notify = bcircles_get_option( 'notify_user_after_removal' );

		if ( empty( $can_notify ) ) {
			return;
		}

		$enabled_notification_types = (array) bcircles_get_option( 'enabled_notification_types' );

		if ( in_array( 'local', $enabled_notification_types, true ) && bp_is_active( 'notifications' ) ) {
			$this->local_notify( $user_id, $circle_id, 'user_deleted' );
		}

		if ( in_array( 'email', $enabled_notification_types, true ) ) {
			$this->email_notify( $user_id, $circle_id, 'user_deleted' );
		}
	}

	/**
	 * Local notify user
	 *
	 * @param int    $user_id User id.
	 * @param int    $circle_id Circle id.
	 * @param string $context Context.
	 */
	public function local_notify( $user_id, $circle_id, $context = '' ) {

		$args = array(
			'user_id'           => $user_id,
			'item_id'           => $circle_id,
			'secondary_item_id' => bp_loggedin_user_id(),
			'component_name'    => 'circle',
			'component_action'  => $context,
		);

		bp_notifications_add_notification( $args );
	}

	/**
	 * Email notify user
	 *
	 * @param int    $user_id User id.
	 * @param int    $circle_id Circle id.
	 * @param string $context Context.
	 */
	public function email_notify( $user_id, $circle_id, $context = '' ) {

		$circle        = bcircles_get_circle( $circle_id );
		$email_to      = bp_core_get_user_email( $user_id );
		$user_name     = bp_core_get_user_displayname( bp_loggedin_user_id() );
		$receiver_name = bp_core_get_user_displayname( $user_id );
		$subject       = '';
		$message       = '';

		if ( 'user_added' == $context ) {
			$subject = bcircles_get_option( 'user_added_to_circle_email_subject' );
			$message = bcircles_get_option( 'user_added_to_circle_email_message' );
		} elseif ( 'user_deleted' == $context ) {
			$subject = bcircles_get_option( 'user_removed_from_circle_email_subject' );
			$message = bcircles_get_option( 'user_removed_from_circle_email_message' );
		}

		if ( empty( $message ) ) {
			return;
		}

		$subject = str_replace( '{user_name}', $user_name, $subject );
		$message = str_replace(
			array( '{receiver_name}', '{user_name}', '{circle_name}' ),
			array( $receiver_name, $user_name, $circle->label ),
			$message
		);

		wp_mail( $email_to, $subject, $message, array() );
	}

	/**
	 * Create default circles.
	 *
	 * @param int $user_id User id.
	 */
	private function create_default_circles( $user_id ) {

		$create_default = bcircles_get_option( 'create_default' );
		$privacy        = bcircles_get_option( 'default_circles_privacy', 'public' );

		if ( empty( $create_default ) ) {
			return;
		}

		$default_circles = bcircles_get_default_circles();

		foreach ( $default_circles as $label ) {
			bcircles_create_circle(
				array(
					'label'   => $label,
					'user_id' => $user_id,
					'privacy' => $privacy,
				)
			);
		}
	}

	/**
	 * Add default circle on account create
	 *
	 * @param int $user_id Activated user id.
	 */
	public function add_circles_on_account_activation( $user_id ) {

		if ( $this->enable_default_circles_creation_on_registration() ) {
			return;
		}

		$this->create_default_circles( $user_id );
	}

	/**
	 * Add default circle on account create
	 *
	 * @param int $user_id Activated user id.
	 */
	public function add_circles_on_registration( $user_id ) {

		if ( ! $this->enable_default_circles_creation_on_registration() ) {
			return;
		}

		$this->create_default_circles( $user_id );
	}

	/**
	 * Circle cleanup on account deleted
	 *
	 * @param int $user_id User id.
	 */
	public function on_account_delete( $user_id ) {

		$circles = bcircles_get_user_circles( $user_id, false, false );

		foreach ( $circles as $circle ) {
			bcircles_delete_circle( $circle );
		}

		bcircles_remove_user( $user_id );
	}

	/**
	 * Mark notification read
	 */
	public function clear_notifications() {

		if ( ! bp_is_active( 'notifications' ) || ! bp_is_notifications_component() ) {
			return;
		}

		if ( empty( $_GET['action'] ) || empty( $_GET['nonce'] ) || empty( $_GET['notification_id'] ) || 'bcircles_clear' != $_GET['action'] ) {
			return;
		}

		if ( ! bp_is_my_profile() || ! wp_verify_nonce( $_GET['nonce'], 'bcircles_clear_notification' ) ) {
			return;
		}

		$notification_id = absint( $_GET['notification_id'] );

		bp_notifications_mark_notification( $notification_id, 0 );
	}

	/**
	 * Load add user to circle template
	 */
	public function load_add_user_template() {

		if ( ! is_user_logged_in() ) {
			return;
		}

		require_once bcircles_locate_template( array( 'modal/add-circle-user.php' ) );
	}

	/**
	 * Default circles to be created on registration yes or no
	 *
	 * @return bool
	 */
	private function enable_default_circles_creation_on_registration() {
		return apply_filters( 'bcircles_enable_default_circles_creation_on_registration', true );
	}
}