<?php
/**
 * Ajax request handler
 *
 * @package BuddyCircles
 * @subpackage Handler
 */
namespace PressThemes\BuddyCircles\Handlers;

// Exit if file accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

use PressThemes\BuddyCircles\Models\Circle;

/**
 * Class Ajax_Request_Handler
 */
class Ajax_Request_Handler {

	/**
	 * Boot itself
	 */
	public static function boot() {
		$self = new self();
		$self->setup();
	}

	/**
	 * Setup
	 */
	private function setup() {
		add_action( 'wp_ajax_circle_add_circle', array( $this, 'add_circle' ) );
		add_action( 'wp_ajax_circle_modal_add_circle', array( $this, 'modal_add_circle' ) );
		add_action( 'wp_ajax_bcircles_delete_circle', array( $this, 'delete_circle' ) );
		add_action( 'wp_ajax_circle_update_circle', array( $this, 'update_circle' ) );
		add_action( 'wp_ajax_bcircles_add_user', array( $this, 'add_user' ) );
		add_action( 'wp_ajax_bcircles_delete_user', array( $this, 'delete_user' ) );
	}

	/**
	 * Add circle
	 */
	public function add_circle() {

		check_ajax_referer( 'bcircles_action', 'nonce', true );

		if ( ! bcircles_user_can_create_circle( get_current_user_id() ) ) {
			wp_send_json_error( array( 'message' => __( 'Invalid action.', 'buddycircles' ) ) );
		}

		$label     = isset( $_POST['label'] ) ? sanitize_text_field( wp_unslash( $_POST['label'] ) ) : '';
		$logged_id = bp_loggedin_user_id();
		$privacy   = isset( $_POST['privacy'] ) ? sanitize_text_field( wp_unslash( $_POST['privacy'] ) ) : 'public';

		if ( ! $label ) {
			wp_send_json_error( array( 'message' => __( 'Please make sure to provide a name for the circle.', 'buddycircles' ) ) );
		}

		if ( bcircles_is_label_duplicate( array(
			'user_id' => $logged_id,
			'label'   => $label,
		) ) ) {
			wp_send_json_error( array( 'message' => __( 'You already have a circle with this name.', 'buddycircles' ) ) );
		}

		$circle = bcircles_create_circle( array(
			'user_id' => $logged_id,
			'label'   => $label,
			'privacy' => $privacy,
		) );

		if ( ! $circle ) {
			wp_send_json_error( array( 'message' => __( 'There was a problem in adding the circle. Please try again later.', 'buddycircles' ) ) );
		}

		$entry = $this->get_li_content( $circle );

		wp_send_json_success( array(
			'message' => __( 'Circle created.', 'buddycircles' ),
			'html'    => "<li><label><input type='checkbox' value='{$circle->id}'>{$circle->label}</label></li>",
			'entry'   => $entry,
			'circle'  => array(
				'id'      => $circle->id,
				'label'   => $circle->label,
				'user_id' => $circle->user_id,
				'privacy' => $circle->privacy,
			),
		) );
	}

	/**
	 * Add circle
	 */
	public function modal_add_circle() {

		check_ajax_referer( 'bcircles_action', 'nonce', true );

		if ( ! bcircles_user_can_create_circle( get_current_user_id() ) ) {
			wp_send_json_error( array( 'message' => __( 'Invalid action.', 'buddycircles' ) ) );
		}

		$label     = isset( $_POST['label'] ) ? sanitize_text_field( wp_unslash( $_POST['label'] ) ) : '';
		$logged_id = bp_loggedin_user_id();
		$privacy   = isset( $_POST['privacy'] ) ? sanitize_text_field( wp_unslash( $_POST['privacy'] ) ) : 'public';

		if ( ! $label ) {
			wp_send_json_error( array( 'message' => __( 'Please make sure to provide a name for the circle.', 'buddycircles' ) ) );
		}

		if ( bcircles_is_label_duplicate( array(
			'user_id' => $logged_id,
			'label'   => $label,
		) ) ) {
			wp_send_json_error( array( 'message' => __( 'You already have a circle with this name.', 'buddycircles' ) ) );
		}

		$circle = bcircles_create_circle( array(
			'user_id' => $logged_id,
			'label'   => $label,
			'privacy' => $privacy,
		) );

		if ( ! $circle ) {
			wp_send_json_error( array( 'message' => __( 'There was a problem in adding the circle. Please try again later.', 'buddycircles' ) ) );
		}

		$entry = $this->get_li_content( $circle );

		wp_send_json_success( array(
			'message' => __( 'Circle created.', 'buddycircles' ),
			'html'    => "<li class='bcircle-list-entry bcircle-not-selected'><label><input type='checkbox' value='{$circle->id}'>{$circle->label}</label></li>",
			'entry'   => $entry,
			'circle'  => array(
				'id'      => $circle->id,
				'label'   => $circle->label,
				'user_id' => $circle->user_id,
				'privacy' => $circle->privacy,
			),
		) );
	}

	/**
	 * Update circle
	 */
	public function update_circle() {

		if ( ! isset( $_POST['_wpnonce'] ) || ! wp_verify_nonce( $_POST['_wpnonce'], 'bcircles_action' ) ) {
			wp_send_json_error( array( 'message' => __( 'Invalid action! Are you sure about it?', 'buddycircles' ) ) );
		}

		$circle_id = isset( $_POST['circle_id'] ) ? absint( $_POST['circle_id'] ) : '';
		$label     = isset( $_POST['label'] ) ? sanitize_text_field( wp_unslash( $_POST['label'] ) ) : '';
		$privacy   = isset( $_POST['privacy'] ) ? sanitize_text_field( wp_unslash( $_POST['privacy'] ) ) : 'public';

		$circle = bcircles_get_circle( $circle_id );

		if ( ! $label || ! $circle->id ) {
			wp_send_json_error( array(
				'message' => __( 'Some detail is missing', 'buddycircles' ),
			) );
		}

		if ( ! bcircles_user_can_edit_circle( get_current_user_id(), $circle ) ) {
			wp_send_json_error( array(
				'message' => __( "You don't have permission to edit this.", 'buddycircles' ),
			) );
		}

		if ( bcircles_is_label_duplicate( array(
			'user_id' => $circle->user_id,
			'label'   => $label,
			'id'      => $circle->id,
		) ) ) {
			wp_send_json_error( array(
				'message' => __( 'Please make sure to provide a unique name for the circle.', 'buddycircles' ),
			) );
		}

		$update = bcircles_update_circle( array(
			'user_id' => $circle->user_id,
			'label'   => $label,
			'id'      => $circle->id,
			'privacy' => $privacy,
		) );

		if ( ! $update ) {
			wp_send_json_error( array(
				'message' => __( 'There was a problem in updating the circle. Please try again later.', 'buddycircles' ),
			) );
		}

		$circle = bcircles_get_circle( $circle->id );
		wp_send_json_success( array(
			'message' => __( 'Updated successfully.', 'buddycircles' ),
			'html'    => $this->get_li_content( $circle ),
		) );
	}

	/**
	 * Delete circle
	 */
	public function delete_circle() {

		if ( ! isset( $_POST['_wpnonce'] ) || ! wp_verify_nonce( $_POST['_wpnonce'], 'bcircles_action' ) ) {
			wp_send_json_error( array(
				'message' => __( 'Invalid action!', 'buddycircles' ),
			) );
		}

		$circle_id = isset( $_POST['circle_id'] ) ? absint( $_POST['circle_id'] ) : '';

		$circle = bcircles_get_circle( $circle_id );

		if ( ! $circle->id ) {
			wp_send_json_error( array(
				'message' => __( 'Invalid action.', 'buddycircles' ),
			) );
		}

		if ( ! bcircles_user_can_delete_circle( bp_loggedin_user_id(), $circle ) ) {
			wp_send_json_error( array(
				'message' => __( "You don't have permission to delete this.", 'buddycircles' ),
			) );
		}

		if ( ! bcircles_delete_circle( $circle ) ) {
			wp_send_json_error( array(
				'message' => __( 'There was a problem in deleting the circle. Please try again later.', 'buddycircles' ),
			) );
		}

		wp_send_json_success( array(
			'message' => __( 'Deleted successfully.', 'buddycircles' ),
			'url'     => bcircles_get_circles_screen_url(),
		) );
	}

	/**
	 * Add User to circle
	 */
	public function add_user() {

		if ( ! isset( $_POST['_wpnonce'] ) || ! wp_verify_nonce( $_POST['_wpnonce'], 'bcircles_action' ) ) {
			wp_send_json_error( array(
				'message' => __( 'Invalid action! Are you sure about it?', 'buddycircles' ),
			) );
		}

		$circle_id = isset( $_POST['circle_id'] ) ? absint( $_POST['circle_id'] ) : 0;
		$user_id   = isset( $_POST['user_id'] ) ? absint( $_POST['user_id'] ) : 0;

		if ( ! $circle_id || ! $user_id ) {
			wp_send_json_error( array(
				'message' => __( 'There is something wrong. Please try again later.', 'buddycircles' ),
			) );
		}

		if ( ! bcircles_user_can_add_user( bp_loggedin_user_id(), $circle_id ) ) {
			wp_send_json_error( array(
				'message' => __( "You don't have permission to add user.", 'buddycircles' ),
			) );
		}

		$circle = bcircles_add_circle_user( $user_id, $circle_id );

		if ( ! $circle ) {
			wp_send_json_error( array(
				'message' => __( 'There was a problem in adding to the circle. Please try again later.', 'buddycircles' ),
			) );
		}

		wp_cache_delete( "circle_{$circle_id}_users", 'bcircles' );

		wp_send_json_success( array(
			'message' => __( 'User added.', 'buddycircles' ),
		) );
	}

	/**
	 * Delete user
	 */
	public function delete_user() {

		if ( ! isset( $_POST['_wpnonce'] ) || ! wp_verify_nonce( $_POST['_wpnonce'], 'bcircles_action' ) ) {
			wp_send_json_error( array(
				'message' => __( 'Invalid action! Are you sure about it?', 'buddycircles' ),
			) );
		}

		$circle_id = isset( $_POST['circle_id'] ) ? absint( $_POST['circle_id'] ) : 0;
		$user_id   = isset( $_POST['user_id'] ) ? absint( $_POST['user_id'] ) : 0;

		if ( ! $circle_id || ! $user_id ) {
			wp_send_json_error( array(
				'message' => __( 'There is something wrong. Please try again later.', 'buddycircles' ),
			) );
		}

		if ( ! bcircles_user_can_remove_user( bp_loggedin_user_id(), $circle_id, $user_id ) ) {
			wp_send_json_error( array(
				'message' => __( "You don't have permission to remove user.", 'buddycircles' ),
			) );
		}

		$circle = bcircles_remove_circle_user( $user_id, $circle_id );

		if ( ! $circle ) {
			wp_send_json_error( array(
				'message' => __( 'There was a problem in removing from the circle. Please try again later.', 'buddycircles' ),
			) );
		}

		wp_cache_delete( "circle_{$circle_id}_users", 'bcircles' );

		wp_send_json_success( array(
			'message' => __( 'User removed.', 'buddycircles' ),
		) );
	}

	/**
	 * Get circle li
	 *
	 * @param Circle $circle Circle.
	 *
	 * @return string
	 */
	public function get_circle_li( $circle ) {

		if ( is_numeric( $circle ) ) {
			$circle = bcircles_get_circle( $circle );
		}

		$privacies = bcircles_get_privacies();

		ob_start();

		?>
        <?php // echo $this->get_li_content( $circle ); ?>

	    <?php
		$content = ob_get_clean();
		return $content;
	}

	/**
     * Get li content
     *
	 * @param Circle $circle Circle object.
	 *
	 * @return string
	 */
	public function get_li_content( $circle ) {

		$privacies = bcircles_get_privacies();

	    ob_start();
		require bcircles_locate_template( array( 'circle-entry.php' ) );
        return ob_get_clean();
    }
}