<?php
/**
 * Model class for circle
 *
 * @package BuddyCircles
 * @subpackage Models
 */

namespace PressThemes\BuddyCircles\Models;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit( 0 );
}

use PressThemes\BuddyCircles\Schema\Schema;

/**
 * Class Circle_Members
 *
 * @property-read int $id circle membership id
 * @property-read int $circle_id circle id.
 * @property-read int $user_id user id.
 */
class Circle_Member {

	/**
	 * Id
	 *
	 * @var int
	 */
	private $id;

	/**
	 * Circle id
	 *
	 * @var int
	 */
	private $circle_id;

	/**
	 * User id added to circle
	 *
	 * @var int
	 */
	private $user_id;

	/**
	 * The constructor.
	 *
	 * @param array $args Args.
	 */
	public function __construct( $args = array() ) {

		if ( is_array( $args ) ) {
			$this->fill( $args );
		} elseif ( is_numeric( $args ) ) {
			$this->with_id( $args );
		}
	}

	/**
	 * Get a property dynamically.
	 *
	 * @param string $name property name.
	 *
	 * @return mixed|null
	 */
	public function __get( $name ) {
		return isset( $this->{$name} ) ? $this->{$name} : null;
	}

	/**
	 * Check if property exists.
	 *
	 * @param string $name property name.
	 *
	 * @return bool
	 */
	public function __isset( $name ) {
		return property_exists( $this, $name );
	}

	/**
	 * Fill class properties
	 *
	 * @param array $args Args.
	 *
	 * @return Circle_Member
	 */
	public function fill( $args ) {

		if ( isset( $args['id'] ) ) {
			$this->id = absint( $args['id'] );
		}

		if ( isset( $args['circle_id'] ) ) {
			$this->circle_id = $args['circle_id'];
		}

		if ( isset( $args['user_id'] ) ) {
			$this->user_id = absint( $args['user_id'] );
		}

		return $this;
	}

	/**
	 * Save
	 *
	 * @return bool
	 */
	public function save() {
		global $wpdb;

		$table = Schema::table( 'members' );

		$data = array(
			'circle_id' => $this->circle_id,
			'user_id'   => $this->user_id,
		);

		$data_format = array( '%d', '%d' );

		if ( ! empty( $this->id ) ) {
			$save = $wpdb->update( $table, $data, array( 'id' => $this->id ), $data_format, '%d' );
		} else {
			$save = $wpdb->insert( $table, $data, $data_format );
			$this->id = $wpdb->insert_id;
		}

		if ( $save ) {
			return true;
		}

		return false;
	}

	/**
	 * Delete
	 *
	 * @return bool
	 */
	public function delete() {
		return self::delete_by_id( $this->id );
	}

	/**
	 * Delete by id
	 *
	 * @param int $id Id.
	 *
	 * @return bool
	 */
	public static function delete_by_id( $id ) {
		global $wpdb;

		$table = Schema::table( 'members' );

		$delete = $wpdb->delete( $table, array( 'id' => $id ), '%d' );

		if ( $delete ) {
			return true;
		}

		return false;
	}

	/**
	 * Get An instance of this from database
	 *
	 * @param int $id Id.
	 *
	 * @return Circle_Member|bool
	 */
	private function with_id( $id ) {

		global $wpdb;

		$table = Schema::table( 'members' );

		$row = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$table} WHERE id = %d", $id ), 'ARRAY_A' );

		if ( empty( $row ) ) {
			return false;
		}

		return $this->fill( $row );
	}

	/**
	 * Get engaged circles by arguments
	 *
	 * @param array $args Args.
	 *
	 * @return  Circle_Member[]
	 */
	public static function get_by( $args = array() ) {

		global $wpdb;

		$sql             = '';
		$table           = Schema::table( 'members' );
		$where_condition = array();
		$order_by_sql    = '';

		if ( empty( $args ) ) {
			return false;
		}

		if ( isset( $args['id'] ) ) {
			$where_condition[] = $wpdb->prepare( 'id = %d', $args['id'] );
		}

		if ( isset( $args['user_id'] ) ) {
			$where_condition[] = $wpdb->prepare( 'user_id = %d', $args['user_id'] );
		}

		if ( isset( $args['circle_id'] ) ) {

			if ( is_array( $args['circle_id'] ) ) {
				$list              = '(' . join( ',', $args['circle_id'] ) . ')';
				$where_condition[] = "circle_id IN {$list}";
			} else {
				$where_condition[] = $wpdb->prepare( 'circle_id = %d', $args['circle_id'] );
			}
		}

		if ( isset( $args['orderby'] ) ) {
			$order_by_sql = $wpdb->prepare( 'ORDER BY %s', $args['orderby'] );

			if ( isset( $args['sort_order'] ) && in_array( $args['sort_order'], array( 'ASC', 'DESC' ), true ) ) {
				$order_by_sql = $order_by_sql . ' ' . $args['sort_order'];
			} else {
				$order_by_sql = $order_by_sql . ' DESC';
			}
		}

		$sql = "SELECT * FROM {$table} WHERE " . join( ' AND ', $where_condition ) . ' ' . $order_by_sql;

		return $wpdb->get_results( $sql );
	}

	/**
	 * Get user in user circles
	 *
	 * @param int $user_id User id.
	 *
	 * @return array|bool|mixed
	 */
	public static function get_user_circles_users( $user_id ) {
		global $wpdb;

		$user_table    = Schema::table( 'members' );
		$circles_table = Schema::table( 'circles' );
		$uu = array();

		$query = "SELECT circle_id, user_id FROM {$user_table}  WHERE circle_id IN( SELECT id FROM {$circles_table} WHERE user_id = %d )";

		$query   = $wpdb->prepare( $query, $user_id );
		$results = $wpdb->get_results( $query );

		foreach ( $results as $result ) {
			$circles = isset( $uu[ $result->user_id ] ) ? $uu[ $result->user_id ] : array();
			array_push( $circles, $result->circle_id );
			$uu[ $result->user_id ] = array_unique( $circles );
		}

		return $uu;
	}

	/**
	 * Deleted by argument
	 *
	 * @param array $args Args.
	 *
	 * @return bool
	 */
	public static function delete_by( $args = array() ) {

		global $wpdb;

		$sql             = '';
		$table           = Schema::table( 'members' );
		$where_condition = array();

		if ( empty( $args ) ) {
			return false;
		}

		if ( isset( $args['id'] ) ) {
			$where_condition[] = $wpdb->prepare( 'id = %d', $args['id'] );
		}

		if ( isset( $args['user_id'] ) ) {
			$where_condition[] = $wpdb->prepare( 'user_id = %d', $args['user_id'] );
		}

		if ( isset( $args['circle_id'] ) ) {
			$where_condition[] = $wpdb->prepare( 'circle_id = %d', $args['circle_id'] );
		}

		$sql = "DELETE FROM {$table} WHERE " . join(" AND ", $where_condition );

		$wpdb->query( $sql );

		return true;
	}

	/**
	 * Get the circles created by $user_id of which the $members_id is a member
	 *
	 * @param int $user_id User id.
	 * @param int $member_id Member id.
	 *
	 * @return array|null|object
	 */
	public static function get_member_circles( $user_id, $member_id ) {
		global $wpdb;

		$user_table    = Schema::table( 'members' );
		$circles_table = Schema::table( 'circles' );

		$query = $wpdb->prepare( "SELECT * FROM {$circles_table} WHERE user_id = %d AND id IN ( SELECT circle_id FROM {$user_table} WHERE user_id = %d ) ", $user_id, $member_id );

		$circles = $wpdb->get_results( $query );

		return $circles;
	}

}