<?php
/**
 * Model class for circle
 *
 * @package BuddyCircles
 * @subpackage Models
 */

namespace PressThemes\BuddyCircles\Models;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit( 0 );
}

use PressThemes\BuddyCircles\Schema\Schema;

/**
 * Class BuddyDev_Circle
 *
 * @method Circle set_id($id)
 * @method Circle set_label($label)
 * @method Circle set_privacy( $privacy)
 * @method Circle set_user_id( $privacy)
 *
 * @property int    $id circle id.
 * @property int    $user_id circle creator id.
 * @property string $label circle name.
 * @property string $privacy circle privacy.
 *
 */
class Circle {

	/**
	 * Circle id
	 *
	 * @var int
	 */
	private $id;

	/**
	 * Circle create by user
	 *
	 * @var int
	 */
	private $user_id;

	/**
	 * Circle name
	 *
	 * @var string
	 */
	private $label;

	/**
	 * Circle privacy
	 *
	 * @var string
	 */
	private $privacy = 'public';

	/**
	 * The constructor.
	 *
	 * @param array $args Args.
	 */
	public function __construct( $args = array() ) {

		if ( is_array( $args ) ) {
			$this->fill( $args );
		} elseif ( is_numeric( $args ) ) {
			$this->with_id( $args );
		}
	}

	/**
	 * Fill class properties
	 *
	 * @param array $args Args.
	 *
	 * @return Circle
	 */
	private function fill( $args ) {

		if ( isset( $args['id'] ) ) {
			$this->id = absint( $args['id'] );
		}

		if ( isset( $args['user_id'] ) ) {
			$this->user_id = absint( $args['user_id'] );
		}

		if ( isset( $args['label'] ) ) {
			$this->label = $args['label'];
		}

		if ( isset( $args['privacy'] ) ) {
			$this->privacy = $args['privacy'];
		}

		return $this;
	}

	/**
	 * Set the field value
	 *
	 * @param string $key Field name.
	 * @param mixed  $value Field value.
	 */
	public function __set( $key, $value ) {

		if ( property_exists( $this, $key ) ) {
			$this->{$key} = $value;
		}
	}

	/**
	 * Get the field value
	 *
	 * @param string $key Field name.
	 *
	 * @return mixed|null
	 */
	public function __get( $key ) {

		if ( property_exists( $this, $key ) ) {
			return $this->{$key};
		}

		return null;
	}

	/**
	 * Check if a property exists.
	 *
	 * @param string $name property name.
	 *
	 * @return bool
	 */
	public function __isset( $name ) {
		return property_exists( $this, $name );
	}

	/**
	 * Save circle
	 *
	 * @return bool
	 */
	public function save() {
		global $wpdb;

		$table = Schema::table( 'circles' );

		$data = array(
			'label'   => $this->label,
			'user_id' => $this->user_id,
			'privacy' => $this->privacy,
		);

		$data_format = array( '%s', '%d', '%s' );

		if ( ! empty( $this->id ) ) {
			$save = $wpdb->update( $table, $data, array( 'id' => $this->id ), $data_format, '%d' );
		} else {
			$save = $wpdb->insert( $table, $data, $data_format );
			$this->id = $wpdb->insert_id;
		}

		if ( $save ) {
			return true;
		}

		return false;
	}

	/**
	 * Delete circle
	 *
	 * @return bool
	 */
	public function delete() {

		$deleted = self::delete_by_id( $this->id );

		do_action( 'bcircles_circle_deleted', $this );

		return $deleted;
	}

	/**
	 * Delete circle by id
	 *
	 * @param int $id Circle id.
	 *
	 * @return bool
	 */
	public static function delete_by_id( $id ) {
		global $wpdb;

		$table = Schema::table( 'circles' );

		$delete = $wpdb->delete( $table, array( 'id' => $id ), '%d' );

		if ( $delete ) {
			return true;
		}

		return false;
	}

	/**
	 * Get An instance of this call from database
	 *
	 * @param int $id Circle id.
	 *
	 * @return $this|bool
	 */
	private function with_id( $id ) {
		global $wpdb;

		$table = Schema::table( 'circles' );

		$row = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$table} WHERE id = %d", $id ), 'ARRAY_A' );

		if ( empty( $row ) ) {
			return false;
		}

		return $this->fill( $row );
	}

	/**
	 * Get circles by arguments
	 *
	 * @param array $args Args.
	 *
	 * @return  Circle[]
	 */
	public static function get_by( $args = array() ) {
		global $wpdb;

		if ( empty( $args ) ) {
			return array();
		}

		$sql   = '';
		$table = Schema::table( 'circles' );

		$where_conditions = array();

		if ( isset( $args['id'] ) ) {
			$where_conditions[] = $wpdb->prepare( 'id = %d', $args['id'] );
		}

		if ( isset( $args['user_id'] ) ) {
			$where_conditions[] = $wpdb->prepare( 'user_id = %d', $args['user_id'] );
		}

		if ( ! empty( $args['privacy'] ) ) {
			$privacy            = join( "','", (array) $args['privacy'] );
			$where_conditions[] = "privacy IN ('$privacy')";
		}

		$order_by_sql = '';

		if ( isset( $args['orderby'] ) ) {
			$order_by_sql = $wpdb->prepare( 'ORDER BY %s', $args['orderby'] );

			if ( isset( $args['sort_order'] ) && in_array( $args['sort_order'], array( 'ASC', 'DESC' ), true ) ) {
				$order_by_sql = $order_by_sql . ' ' . $args['sort_order'];
			} else {
				$order_by_sql = $order_by_sql . ' DESC';
			}
		}

		if ( ! empty( $args['fields'] ) ) {
			$select = "SELECT {$args['fields']} ";
		} else {
			$select = "SELECT *";
		}

		$sql = "{$select} FROM {$table} WHERE " . join( ' AND ', $where_conditions ) . ' ' . $order_by_sql;

		return $wpdb->get_results( $sql );
	}

	/**
	 * Deleted by arguments
	 *
	 * @param array $args Args.
	 *
	 * @return bool
	 */
	public static function delete_by( $args = array() ) {
		global $wpdb;

		if ( empty( $args ) ) {
			return false;
		}

		$sql = '';
		$table = Schema::table( 'circles' );

		$where_conditions = array();

		if ( isset( $args['id'] ) ) {
			$where_conditions[] = $wpdb->prepare( 'id = %d', $args['id'] );
		}

		if ( isset( $args['user_id'] ) ) {
			$where_conditions[] = $wpdb->prepare( 'user_id = %d', $args['user_id'] );
		}

		if ( ! empty( $args['privacy'] ) ) {
			$privacy            = join( "','", (array) $args['privacy'] );
			$where_conditions[] = "privacy IN ('$privacy')";
		}

		$sql = "DELETE FROM {$table} WHERE " . join( ' AND ', $where_conditions );

		if ( $wpdb->query( $sql ) ) {
			return true;
		}

		return false;
	}

	/**
	 * Check if same label duplicate or not
	 *
	 * @param array $args Args.
	 *
	 * @return bool
	 */
	public static function is_label_duplicate( $args ) {
		global $wpdb;

		$where_conditions = array();
		$table            = Schema::table( 'circles' );
		$sql              = '';

		if ( isset( $args['id'] ) ) {
			$where_conditions[] = $wpdb->prepare( 'id !=%d', $args['id'] );
		}

		if ( isset( $args['user_id'] ) ) {
			$where_conditions[] = $wpdb->prepare( 'user_id = %d', $args['user_id'] );
		}

		if ( isset( $args['label'] ) ) {
			$where_conditions[] = $wpdb->prepare( 'label = %s', $args['label'] );
		}

		if ( ! empty( $where_conditions ) ) {
			$sql = join( ' AND ', $where_conditions );
		}

		$results = $wpdb->get_results( "SELECT * FROM {$table} WHERE {$sql}" );

		if ( ! empty( $results ) ) {
			return true;
		}

		return false;
	}

	/**
	 * Check for existence
	 *
	 * @param array $args Args.
	 *
	 * @return bool
	 */
	public static function exists( $args ) {
		global $wpdb;

		$where_conditions = array();
		$sql              = '';
		$table            = Schema::table( 'circles' );

		if ( isset( $args['id'] ) ) {
			$where_conditions[] = $wpdb->prepare( 'id =%d', $args['id'] );
		}

		if ( isset( $args['user_id'] ) ) {
			$where_conditions[] = $wpdb->prepare( 'user_id = %d', $args['user_id'] );
		}

		if ( isset( $args['label'] ) ) {
			$where_conditions[] = $wpdb->prepare( 'label = %s', $args['label'] );
		}

		if ( ! empty( $where_conditions ) ) {
			$sql = join( ' AND ', $where_conditions );
		}

		$results = $wpdb->get_results( "SELECT * FROM {$table} WHERE {$sql}" );

		if ( ! empty( $results ) ) {
			return true;
		}

		return false;
	}

	/**
	 * Get total
	 *
	 * @param array $args Args.
	 *
	 * @return null|string
	 */
	public static function get_total( $args ) {
		global $wpdb;

		$where_conditions = array();
		$sql              = '';
		$table            = Schema::table( 'circles' );

		if ( isset( $args['id'] ) ) {
			$where_conditions[] = $wpdb->prepare( 'id =%d', $args['id'] );
		}

		if ( isset( $args['user_id'] ) ) {
			$where_conditions[] = $wpdb->prepare( 'user_id = %d', $args['user_id'] );
		}

		if ( isset( $args['label'] ) ) {
			$where_conditions[] = $wpdb->prepare( 'label = %s', $args['label'] );
		}

		if ( ! empty( $args['privacy'] ) ) {
			$privacy            = join( "','", (array) $args['privacy'] );
			$where_conditions[] = "privacy IN ('$privacy')";
		}

		if ( ! empty( $where_conditions ) ) {
			$sql = join( ' AND ', $where_conditions );
		}

		return $wpdb->get_var( "SELECT COUNT('*') FROM {$table} WHERE {$sql}" );
	}

	public function as_plain() {
		return (object) get_object_vars( $this );
	}
}
