<?php
/**
 * Permissions functions
 *
 * @package BuddyCircles
 */

// Exit if file accessed directly.
use PressThemes\BuddyCircles\Models\Circle;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Can circle circles
 *
 * @param int $user_id user it to check for.
 *
 * @return bool
 */
function bcircles_user_can_create_circle( $user_id = 0 ) {

	if ( ! $user_id ) {
		$user_id = get_current_user_id();
	}

	if ( ! $user_id ) {
		return false;
	}

	if ( is_super_admin( $user_id ) ) {
		return true;
	}

	$can_create    = false;
	$allowed_roles = (array) bcircles_get_option( 'who_can_create' );

	if ( in_array( 'all', $allowed_roles, true ) ) {
		$can_create = true;
	} else {
		$user      = wp_get_current_user();
		$has_roles = array_intersect( $user->roles, $allowed_roles );

		if ( ! empty( $has_roles ) ) {
			$can_create = true;
		}
	}

	return (bool) apply_filters( 'bcircles_user_can_create_circle', $can_create, $user_id );
}

/**
 * Check if user can delete circle
 *
 * @param int    $user_id User id.
 * @param Circle $circle Circle object.
 *
 * @return bool
 */
function bcircles_user_can_delete_circle( $user_id, $circle ) {
	$can = false;

	if ( is_super_admin( $user_id ) || ( ! empty( $circle->user_id ) && $user_id == $circle->user_id ) ) {
		$can = true;
	}

	return (bool) apply_filters( 'bcircles_user_can_delete_circle', $can, $user_id, $circle );
}

/**
 * Check if user can edit circle
 *
 * @param int    $user_id User id.
 * @param Circle $circle Circle object.
 *
 * @return bool
 */
function bcircles_user_can_edit_circle( $user_id, $circle ) {

	if ( is_numeric( $circle ) ) {
		$circle = bcircles_get_circle( $circle );
	}

	$can = false;

	if ( is_super_admin( $user_id ) || ( ! empty( $circle->user_id ) && $user_id == $circle->user_id ) ) {
		$can = true;
	}

	return (bool) apply_filters( 'bcircles_user_can_edit_circle', $can, $user_id, $circle );
}

/**
 * Check if user can add user
 *
 * @param int        $user_id User id.
 * @param int|Circle $circle Circle id or object.
 *
 * @return bool
 */
function bcircles_user_can_add_user( $user_id, $circle ) {

	if ( is_numeric( $circle ) ) {
		$circle = bcircles_get_circle( $circle );
	}

	$can = false;

	if ( is_super_admin( $user_id ) || ( ! empty( $circle->user_id ) && $user_id == $circle->user_id ) ) {
		$can = true;
	}

	return (bool) apply_filters( 'bcircles_user_can_add_user', $can, $user_id, $circle );
}

/**
 * Check if user can delete user
 *
 * @param int        $user_id          User id.
 * @param int|Circle $circle           Circle id or object.
 * @param int        $removing_user_id User going to remove.
 *
 * @return bool
 */
function bcircles_user_can_remove_user( $user_id, $circle, $removing_user_id = 0 ) {

	if ( is_numeric( $circle ) ) {
		$circle = bcircles_get_circle( $circle );
	}

	$can = false;

	if ( is_super_admin( $user_id ) || ( ! empty( $circle->user_id ) && $user_id == $circle->user_id ) ) {
		$can = true;
	}

	return (bool) apply_filters( 'bcircles_user_can_remove_user', $can, $user_id, $circle, $removing_user_id );
}

/**
 * Check if user can view circle members
 *
 * @param int $user_id User id to check if has permission or not.
 * @param int $circle_id Circle id.
 *
 * @return bool
 */
function bcircles_user_can_view_circle( $user_id, $circle_id ) {

	if ( empty( $user_id ) ) {
		$user_id = bp_loggedin_user_id();
	}

	if ( is_super_admin() ) {
		return true;
	}

	$can     = false;
	$privacy = (array) bcircles_get_option( 'privacy' );
	$circle  = bcircles_get_circle( $circle_id );

	if ( ! $circle || ! in_array( $circle->privacy, $privacy, true ) ) {
		return $can;
	}

	if ( $user_id == $circle->user_id ) {
		// Fix so that user can not see own circles on other user screen.
		$can = bp_is_my_profile() ? true : false;
	} elseif ( 'public' === $circle->privacy ) {
		$can = true;
	} elseif ( 'friends_only' === $circle->privacy && bp_is_active( 'friends' ) && 'is_friend' == friends_check_friendship_status( $user_id, bp_displayed_user_id() ) ) {
		$can = true;
	}

	return (bool) apply_filters( 'bcircles_user_can_view_circle', $can, $user_id, $circle );
}
